unit GraphQL.Bookshelf;

interface

uses
  SysUtils, Generics.Collections, Rtti, TypInfo,
  GraphQL.Document.Values,
  GraphQL.Executer,
  GraphQL.Schema;

type
  TMutation = class
  public
    function DateToString(date: TDateTime): string;
  end;

  TQuery = class
  private
    function CurrentDate: TDateTime;
  end;

  TSchemaDateTimeType = class(TSchemaScalarType)
  strict protected
    function ParseLiteral(Value: TASTValue): TValue; override;
  public
    constructor Create; reintroduce;
  public
    function ParseValue(const Value: TValue): TValue; override;
    function Serialize(const Value: TValue): TValue; override;
  end;

const
  BookshelfSchema =
    'type Mutation {                                 ' + sLineBreak +
    '  dateToString(date: DateTime!): String!        ' + sLineBreak +
    '}                                               ' + sLineBreak +
    'type Query {                                    ' + sLineBreak +
    '  currentDate(): DateTime!                      ' + sLineBreak +
    '}';

procedure InitBookshelfSchema(Schema: TSchemaDocument);

implementation

uses
  System.DateUtils;

procedure InitBookshelfSchema(Schema: TSchemaDocument);
begin
  Schema.Bind('Query', TQuery);
  Schema.Bind('Mutation', TMutation);
end;

procedure RegisterGlobalTypes(Schema: TSchemaDocument);
begin
  Schema.Add(TSchemaDateTimeType.Create);
end;

{ TMutation }

function TMutation.DateToString(date: TDateTime): string;
begin
  Result := DateToStr(date);
end;

{ TQuery }

function TQuery.CurrentDate: TDateTime;
begin
  Result := Now;
end;

{ TSchemaDateTimeype }

constructor TSchemaDateTimeType.Create;
begin
  inherited Create('DateTime');
end;

function TSchemaDateTimeType.ParseLiteral(Value: TASTValue): TValue;
var
  DateValue: TDateTime;
begin
  if Value is TASTStringValue then
  begin
    if not TryISO8601ToDate(TASTSTringValue(Value).Value, DateValue) then
      raise EGraphQLCoercingParseLiteral.Create(Self.DisplayName, Value);
    Result := TValue.From<TDateTime>(DateValue);
  end
  else
    raise EGraphQLCoercingParseLiteral.Create(Self.DisplayName, Value);
end;

function TSchemaDateTimeType.ParseValue(const Value: TValue): TValue;
begin
  if (Value.TypeInfo =  System.TypeInfo(TDateTime)) or
    (Value.TypeInfo =  System.TypeInfo(TDate)) or
    (Value.TypeInfo =  System.TypeInfo(TTime)) then
    Result := Value
  else
    raise EGraphQLCoercingParseValue.Create(Self.DisplayName, Value);
end;

function TSchemaDateTimeType.Serialize(const Value: TValue): TValue;
begin
  if Value.IsEmpty then
    Exit(TValue.Empty);

  if not Value.TryCast(TypeInfo(TDateTime), Result) then
    raise EGraphQLCoercingSerialize.Create(Self.DisplayName, Value);

  Result := DateToISO8601(Value.AsType<TDateTime>, True);
end;

initialization
  TSchemaDocument.OnGlobalCreate := RegisterGlobalTypes;

finalization

end.
